#include "squid.h"

static int s_iInitCount = 0;
#define VENDOR   "Nobody.org"
#define SOFTWARE "SquidNT"
#define VERSION  "2.3Stable4 beta"
#define REGKEY "SOFTWARE\\" VENDOR "\\" SOFTWARE "\\" VERSION
TCHAR servname[512]=SQUID_SERVICE_NAME;

SERVICE_STATUS svcStatus;

static int create_key(void)
{
    static char *keys[] = 
    { "SOFTWARE",
	VENDOR,
	SOFTWARE,
	VERSION,
	NULL
    };
    int index;
    HKEY hKey;
    HKEY hKeyNext;
    int retval;
    int rv;

    hKey = HKEY_LOCAL_MACHINE;
    index = 0;
    retval = 0;

    /* Walk the tree, creating at each stage if necessary */
    while (keys[index]) {
	unsigned long result;

	rv = RegCreateKeyEx(hKey,
			    keys[index], /* subkey */
			    0,	         /* reserved */
			    NULL,        /* class */
			    REG_OPTION_NON_VOLATILE,
			    KEY_WRITE,
			    NULL,
			    &hKeyNext,
			    &result);
	if (rv != ERROR_SUCCESS) {
	    debug(1,1)("RegCreateKeyEx(%s),%d\n", keys[index],rv);
	    retval = -4;
	}

	/* Close the old key */
	rv = RegCloseKey(hKey);
	if (rv != ERROR_SUCCESS) {
	    debug(1,1)("RegCloseKey %d\n", rv);
	    if (retval == 0) {
		/* Keep error status from RegCreateKeyEx, if any */
		retval = -4;  
	    }
	}

	if (retval) {
	    break;
	}

	hKey = hKeyNext;
	index++;
    }

    if (keys[index] == NULL) {
	/* Close the final key we opened, if we walked the entire
	 * tree
	 */
	rv = RegCloseKey(hKey);
	if (rv != ERROR_SUCCESS) {
	    debug(1,1)("RegCloseKey %d\n", rv);
	    if (retval == 0) {
		/* Keep error status from RegCreateKeyEx, if any */
		retval = -4;  
	    }
	}
    }

    return retval;
}

static int store_key(const char *key, DWORD type, unsigned char *value, int value_size)
{
    long rv;
    HKEY hKey;
    int retval;

    rv = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
		      REGKEY,
		      0,
	 	      KEY_WRITE,
		      &hKey);

    if (rv == ERROR_FILE_NOT_FOUND) {
	/* Key could not be opened -- try to create it 
	 */
	if (create_key() < 0) {
	    /* Creation failed (error already reported) */
	    return -4;
	}
	
	/* Now it has been created we should be able to open it
	 */
	rv = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
		  REGKEY,
		  0,
	 	  KEY_WRITE,
		  &hKey);

	if (rv == ERROR_FILE_NOT_FOUND) {
	    debug(1,1)("Registry does not contain key %s after creation",REGKEY );
	    return -1;
	}
    }

    if (rv != ERROR_SUCCESS) {
	debug(1,1)("RegOpenKeyEx HKLM\\%s, %d\n", REGKEY,rv);
	return -4;
    }

    /* Now set the value and data */
    rv = RegSetValueEx(hKey, 
		       key,	/* value key name */
		       0,	/* reserved */
		       type,	/* type */
		       value,	/* value data */
		       (DWORD)value_size); /* for size of "value" */

    retval = 0;	    /* Return value */

    if (rv != ERROR_SUCCESS) {
	debug(1,1)("RegQueryValueEx(key %s),%d\n",key,rv);
	retval = -4;
    }
    else {
	debug(1,1)("Registry stored HKLM\\%s\\%s value %s\n",
		REGKEY, 
		key,
		type == REG_SZ ? value : (unsigned char*)"(not displayable)");
    }

    /* Make sure we close the key even if there was an error storing
     * the data
     */
    rv = RegCloseKey(hKey);
    if (rv != ERROR_SUCCESS) {
	debug(1,1)("RegCloseKey HKLM\\%s, %d\n", REGKEY,rv);
	if (retval == 0) {
	    /* Keep error status from RegQueryValueEx, if any */
	    retval = -4;  
	}
    }

    return retval;
}

void RemoveService()
{
    SC_HANDLE   schService;
    SC_HANDLE   schSCManager;
    schSCManager = OpenSCManager(
                        NULL,                   // machine (NULL == local)
                        NULL,                   // database (NULL == default)
                        SC_MANAGER_ALL_ACCESS   // access required
                        );
    if (!schSCManager) debug(1,1)("OpenSCManager failed");
    else {
        schService = OpenService(schSCManager, servname, SERVICE_ALL_ACCESS);

        if (schService == NULL) debug(1,1)("OpenService failed");
            /* Could not open the service */
        else {
            /* try to stop the service */
            if (ControlService(schService, SERVICE_CONTROL_STOP, &svcStatus)) {
                Sleep(1000);
                while(QueryServiceStatus(schService, &svcStatus)) {
                    if(svcStatus.dwCurrentState == SERVICE_STOP_PENDING)
                        Sleep(1000);
                    else
                        break;
                }
            }

            // now remove the service
            if (DeleteService(schService) == 0) debug(1,1)("DeleteService failed");
			else printf("Service SquidNT2.3stable4 beta deleted successfuly, see ya'round :)\n");

            CloseServiceHandle(schService);
        }

        CloseServiceHandle(schSCManager);
    }

}

void InstallService()
{
    SC_HANDLE   schService;
    SC_HANDLE   schSCManager;

    TCHAR szPath[512];
	int lenpath;
    if ((lenpath=GetModuleFileName( NULL, szPath, 512 )) == 0)
    {
        exit(1);
        return;
    }

    schSCManager = OpenSCManager(
                        NULL,                   // machine (NULL == local)
                        NULL,                   // database (NULL == default)
                        SC_MANAGER_ALL_ACCESS   // access required
                        );
   if (!schSCManager) debug(1,1)("OpenSCManager failed");
    else {
        schService = CreateService(
            schSCManager,               // SCManager database
            servname,        // name of service
            servname, // name to display
            SERVICE_ALL_ACCESS,         // desired access
            SERVICE_WIN32_OWN_PROCESS,  // service type
            SERVICE_AUTO_START,       // start type
            SERVICE_ERROR_NORMAL,       // error control type
            (const char*)szPath,                     // service's binary
            NULL,                       // no load ordering group
            NULL,                       // no tag identifier
            NULL,       // dependencies
            NULL,                       // LocalSystem account
            NULL);                      // no password

        if (schService) {
            CloseServiceHandle(schService);

	    /* Now store the server_root in the registry */
	    store_key("ServerRoot", REG_SZ,(unsigned char*) szPath, lenpath+1);
		printf("Service SquidNT2.3 stable4 beta installed successfuly, enjoy :)\n");
		printf("that is, u may start it from the Services applet of control panel\n");
		printf("Don't forget to edit squid.conf before starting it\n\n");
		printf("Romeo Anghelache, http://www.andrew.cmu.edu/~romeo/ \n");
		printf("Guido Serassio, http://serassio.interfree.it \n");
		printf("No warranties of any kind, no time for support!\n");
		printf("This work derived from GNU Squid2.3 stable 4\n");
        }
        else  debug(1,1)("CreateService failed");

        CloseServiceHandle(schSCManager);
    }
}

int Win32SockInit(void)
{
    int iVersionRequested;
    WSADATA wsaData;
    int err;

    if (s_iInitCount > 0) {
	s_iInitCount++;
	return (0);
    }
    else if (s_iInitCount < 0)
	return (s_iInitCount);

    /* s_iInitCount == 0. Do the initailization */
    iVersionRequested = MAKEWORD(2, 0);
    err = WSAStartup((WORD) iVersionRequested, &wsaData);
    if (err) {
	s_iInitCount = -1;
	return (s_iInitCount);
    }
    if (LOBYTE(wsaData.wVersion) != 2 ||
	HIBYTE(wsaData.wVersion) != 0) {
	s_iInitCount = -2;
	WSACleanup();
	return (s_iInitCount);
    }
	debug(1,1)("Windows sockets initialised");
    s_iInitCount++;
    return (s_iInitCount);

}


void Win32SockCleanup(void)
{
    if (--s_iInitCount == 0)
	WSACleanup();
    return;
}
